@ --------------------------------------- Ramsey2b.g ---------------------------------------------------------

   Alfred Maussner
   16. November 2003 
   Last revision: 20. February 2004

   Purpose: Solve the deterministic growth model (Ramsey2) from Chapter 1 of Heer/Maussner
            using projection methods. You may choose between least squares (in this case
            we minimize the least squares function integral instead of solving the respective system of
            first order conditions), Galerkin and Chebyshev collocation.

   Notes: In order to get proper screen messages, open the DOS compatibility window
          (Tools->DOS Compatibility Window).

          For the program to run you must either include the names of the procedures
          in the file Toolbox.src in your Gauss user library (user.lcg) or in any other
          Gauss library file or use the statement #include Toolbox.src (if this file
          is in your current working directory).

------------------------------------------------------------------------------------------------------------- @

new;
library user, pgraph, dge;

/* Parameters */
alpha=0.28;
beta=0.994;
delta=0.011;
eta=2;      

/* Stationary Solution */
ks=((alpha*beta)/(1-beta*(1-delta)))^(1/(1-alpha));
cs=(ks^alpha)-delta*ks;

/* Parameters for the algorithm */
n=5;             @ degree of polynomial @
nint=100;        @ number of nodes in Gauss-Chebysehv quadratur @
ku=1.5*ks;       @ upper bound of capital stock                 @
kl=0.5*ks;       @ lower bound of capital stock                 @
d=(kl|ku);
method=2;        @ =1: Least Squares, 2: Galerkin, 3: Collocation @

/* Name of the output file */
outfile="d:\\tbk\\tb5\\g\\ch4\\Ramsey2b.txt";

output file=^outfile on;
Datum;
output off;

if (eta==1) and (delta==1);
   kvec=seqa(kl,(ku-kl)/99,100);
   cvec=(1-alpha*beta)*kvec^alpha;
   avec1=ChebCoef(&c0,n,100,d);  @ Initial Coefficients @
   asec=hsec;
   if method==1; goto LS; endif;
   if method==2; goto GA; endif;
   if method==3; goto CO; endif;

endif;

/* Parameters for the search routine */
Npar=n;
Npop=50;
Ngen=100;

cls;
asec=hsec;

{avec1,fit}=GSearch1(Npar,Npop,Ngen,&f); @ you may want to use this procedure to find acceptable starting values @


LS:
cls;
if method==1;
  /* Refine Solution using our own quasi Newton routine */
  cls;
  _QN_Print=1;
  _QN_GradTol=1.e-10;
  {avec2,crit}=QuasiNewton(avec1,&f);
  output on;
  ?"Exit from Quasi Newton:";
  crit;
  i=1;
  do until i>rows(avec2);
     ?ftos(avec2[i],"*.*lf",10,6);
     i=i+1;
  endo;
  @output off;@wait;
endif;
GA:
cls;
if method==2;
  /* Refine Solution using Galerkin */
  _MNR_Print=1;
  _MNR_Global=1;
  {avec2,crit}=FixvMN1(avec1,&Sys2);
  output on;
  ?"Exit from FixvMN1:";  
  crit;
  i=1;
  do until i>rows(avec2);
     ?ftos(avec2[i],"*.*lf",10,6);
     i=i+1;
  endo;
  output off;wait;
endif;
CO:
cls;
if method==3;
  _MNR_Print=1;
  _MNR_Global=1;
  {avec2,crit}=FixvMN1(avec1,&Sys1);
  output on;
  ?"Exit from FixvMN1:";
  crit;
  i=1;
  do until i>rows(avec2);
     ?ftos(avec2[i],"*.*lf",10,6);
     i=i+1;
  endo;
  output off;
endif;

output on;
?"Run Time: " etstr(hsec-asec);
output off;
plot:
if method==1;text="\000Least Squares";endif;
if method==2;text="\000Galerkin";     endif;
if method==3;text="\000Collocation";  endif;

/* Compare to true solution */
if (eta==1) and (delta==1);
   kvec =seqa(kl,(ku-kl)/99,100);
   cvec1=(1-alpha*beta)*(kvec^alpha);
   cvec2=ChebEval1(avec2,kvec,d);
   output on;
   if method==1;
     ?"Maximum of distance of least squares solution to analytic solution:";
   endif;
   if method==2;
     ?"Maximum of distance of Galerkin solution to analytic solution:";
   endif;
   if method==3;
     ?"Maximum of distance of collocation solution to analytic solution:";
   endif;

   ftos(maxc(abs(cvec1-cvec2)),"*.*lf",10,7);

   output off;

   GraphSettings;
   xlabel("K");
   ylabel("C");
   _plwidth=7;
   _pcolor={0, 0};
   _pltype={6, 3};
   _pstype=8;
   _plctrl={0, 3};
   _psymsiz=2;   
   _plegstr="True Solution"$+text;
   _plegctl=(1~5~minc(kvec)~0.95*maxc(cvec2));
   xy(kvec,cvec1~cvec2);
 else;
   load cvec1000;
   nobs=rows(cvec1000);
   kvec=seqa(kl,(ku-kl)/(nobs-1),nobs);
   cvec2=ChebEval1(avec2,kvec,d);
   output on;
   if method==1;
     ?"Maximum of distance of least squares solution to VI solution:";
   endif;
   if method==2;
     ?"Maximum of distance of Galerkin solution to VI solution:";
   endif;
   if method==3;
     ?"Maximum of distance of collocation solution to VI solution:";
   endif;

   ftos(maxc(abs(cvec1000-cvec2)),"*.*lf",10,7);
   output off;
   GraphSettings;
   xlabel("K");
   ylabel("C");
   _plwidth=7;
   _pcolor={0, 0};
   _pltype={6, 3};
   _pstype=8;
   _plctrl={0, 20};
   _psymsiz=2;   
   _plegstr="VI Solution"$+text;
   _plegctl=(1~5~minc(kvec)~0.95*maxc(cvec2));
   xy(kvec,cvec1000~cvec2);
endif;
   

ende:
end;


/* --------------------------------------------------- Procedures ------------------------------------------------ */

/* c0(k): returns rows(k) times the stationary solution */
proc(1)=c0(k);
 local nobs, i, c;
 nobs=rows(k);
 c=zeros(nobs,1);
 i=1;
 do until i>nobs;
    c[i]=cvec[sumc(kvec.<=k[i])];
       i=i+1;
 endo;
 retp (c);
endp;

/* f(avec) computes the objective function, whose minimizer is the desired solution
**
** from the main program it uses nint, ku, kl, and d  */

proc(1)=f(avec);

   local i, sum, r, kt, k0, k1, c0, c1;

   i=1; sum=0;
   do until i>nint;
      kt = cos(((2*i - 1)/(2*nint))*pi);   @ i-th zero of T_m            @
      k0 = (1/2)*(kt+1)*(ku-kl) + kl;      @ transformed to k in [kl,ku] @
      c0 = ChebEval1(avec,k0,d);
      if c0<0; retp(miss(1,1)); endif;
      k1 = (k0^alpha) + (1-delta)*k0 - c0;
      if ((k1<kl) or (k1>ku)); retp(miss(1,1)); endif;      
      c1 = ChebEval1(avec,k1,d);
      if (c1<0); retp(miss(1,1));endif;
      r  = beta*((c1/c0)^(-eta))*(1-delta+alpha*(k1^(alpha-1)));
      r  = r-1;
      sum=sum+(r*r)*sqrt(1-(kt^2));

        i=i+1;
   endo;
   sum=sum*(pi*(ku-kl))/(2*nint);

  retp(sum);

endp;

/* Sys1(avec) returns the rhs of the system of equations, whose zero defines
** the collocation solution.
**
** From the main program it uses nint, ku, kl, and d
*/



proc(1)=Sys1(avec);

  local  p, r, i, kt, k0, k1, c0, c1;

    p=rows(avec);
    r=zeros(p,1);

   i=1; 
   do until i>p;
      kt   = cos(((2*i - 1)/(2*p))*pi);      @ i-th zero of T_p            @
      k0   = (1/2)*(kt+1)*(ku-kl) + kl;      @ transformed to k in [kl,ku] @
      c0   = ChebEval1(avec,k0,d);
      if c0<0; retp(miss(1,1)); endif;
      k1   = (k0^alpha) + (1-delta)*k0 - c0;
      if ((k1<kl) or (k1>ku)); retp(miss(1,1)); endif;      
      c1   = ChebEval1(avec,k1,d);
      if (c1<0); retp(miss(1,1)); endif;
      r[i] = beta*((c1/c0)^(-eta))*(1-delta+alpha*(k1^(alpha-1)));
      r[i] = r[i]-1;
         i = i+1;
   endo;

 retp(r);

endp;

/* Sys2(avec) returns the rhs of the system of equations, whose zero defines
** the Galerkin solution.
**
** From the main program it uses nint, ku, kl, and d 
*/

proc(1)=Sys2(avec);

   local i, j, p, sum, r, kt, k0, k1, c0, c1, t0, t1, t2;
   p=rows(avec);
   sum=zeros(p,1);

   j=1;
   t0=ones(nint,1);   
   kt=cos(((2*seqa(1,1,nint)-1)/(2*nint))*pi); @ the nint zeros of T_nint @   
   t1=kt;
   
   do until j>p;          
     i=1;

   do until i>nint;      
      k0    = (1/2)*(kt[i]+1)*(ku-kl) + kl;      @ transformed to k in [kl,ku] @
      c0    = ChebEval1(avec,k0,d);
      if c0<0; retp(miss(1,1)); endif;
      k1    = (k0^alpha) + (1-delta)*k0 - c0;
      if ((k1<kl) or (k1>ku)); retp(miss(1,1)); endif;      
      c1    = ChebEval1(avec,k1,d);
      if (c1<0); retp(miss(1,1));endif;
      r     = beta*((c1/c0)^(-eta))*(1-delta+alpha*(k1^(alpha-1)));
      r     = r-1;
      r     = r*t0[i]*sqrt(1-(kt[i]^2));
      sum[j]= sum[j]+r;
           i=i+1;
   endo;
   sum[j]=sum[j]*(pi*(ku-kl))/(2*nint);
   t2=2*(kt.*t1)-t0;
   t0=t1;
   t1=t2;
   j =j+1;
 endo;
   

 retp(sum);

endp;

